library ieee;
use ieee.std_logic_1164.all;

entity Ad9512Syncer is
    generic (
        g_ClkFrequency: positive -- input clock frequency in Hz
    );
    port (
        Clk_ik: in std_logic;
        StartSync_i: in std_logic; -- one pulse starts synchronization
        Trigger_i: in std_logic; -- falling edge of this signals determines synchronization point
        Ad9512Sync_o: out std_logic := '0' -- synchronization pulse for AD9512
    );
end entity;

architecture syn of Ad9512Syncer is

    signal Pulse, GeneratorReset_ra: std_logic;

begin

    -- AD9512 function pin is set as SYNCB - needs negative pulse > 1.5*T_rfClk
    -- (f_rfClk >= 1 MHz) => (1.5*T_rfClk <= 1.5 us)
    
    cAd9512SyncPulseGenerator: entity work.PulseGeneratorTime(syn)
        generic map (
            g_ClkFrequency => g_ClkFrequency,
            g_PulseMinWidthInTime => 2 us
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => '0',
            Signal_i => StartSync_i,
            Pulse_o => Pulse
        );
    
    GeneratorReset_ra <= Trigger_i and (not Pulse);
    
    pSyncPulseGenerator: process(Clk_ik, GeneratorReset_ra) is begin
        if GeneratorReset_ra = '1' then
            Ad9512Sync_o <= '0';
        elsif rising_edge(Clk_ik) then
            if StartSync_i = '1' then
                Ad9512Sync_o <= '1';
            end if;
        end if;
    end process;

end architecture;