library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.wishbone_pkg.all;

package FfpgPkg is

    type t_Ad5600Interface is record
        FrameSynchronization_n: std_logic;
        SerialClock: std_logic;
        SerialData: std_logic;
    end record;
  
    constant c_Ad5600InterfaceInitValue: t_Ad5600Interface := (
        FrameSynchronization_n => '1',
        SerialClock => '0',
        SerialData => '0'
    );
    
    type t_Mode is (
        e_ModeStop,
        e_ModeContinuous,
        e_ModeSingle
    );
    
    -- FFPG SDB description
    constant c_FfpgSdbDevice: t_sdb_device := (
        abi_class     => x"0000",
        abi_ver_major => x"01",
        abi_ver_minor => x"01",
        wbd_endian    => c_sdb_endian_big,
        wbd_width     => x"4", -- 32 bit granularity
        sdb_component => (
            addr_first  => x"0000_0000_0000_0000",
            addr_last   => x"0000_0000_0000_ffff",
            product     => (
                vendor_id => x"0000_0000_0000_CE42",
                device_id => x"3f76563f",
                version   => x"00000001",
                date      => x"20160725",
                name      => "WB-FFPG pulse gen. ")));
    
    -- f_xwb_bridge_manual_sdb(size, sdb_addr)
    -- Note: sdb_addr is the sdb records address relative to the bridge base address
    constant c_FfpgBridgeSdb: t_sdb_bridge := f_xwb_bridge_manual_sdb(x"0001_ffff", x"0000_0000");

    function f_ModeToSlv(Mode: t_Mode) return std_logic_vector;
    function f_SlvToMode(Slv: std_logic_vector(1 downto 0)) return t_Mode;

end package;

package body FfpgPkg is

    function f_ModeToSlv(Mode: t_Mode) return std_logic_vector is
        variable Result: std_logic_vector(1 downto 0);
    begin
        case Mode is
            when e_ModeStop => Result := "00";
            when e_ModeContinuous => Result := "01";
            when e_ModeSingle => Result := "10";
            --when others => Result := "00";
        end case;
        return Result;
    end function;
    
    function f_SlvToMode(Slv: std_logic_vector(1 downto 0)) return t_Mode is
        variable Result: t_Mode;
    begin
        case Slv is
            when "00" => Result := e_ModeStop;
            when "01" => Result := e_ModeContinuous;
            when "10" => Result := e_ModeSingle;
            when others => Result := e_ModeStop;
        end case;
        return Result;
    end function;

end package body;
