--
-- minimal pulse generator
-- (generated pulse can be longer)
--

library ieee;
use ieee.std_logic_1164.all;

entity PulseGenerator is
	generic (
		g_PulseMinWidth: integer := 3 -- multiply of 'Clk_ik' periods
	);
	port (
		Clk_ik: in std_logic;
		Reset_ir: in std_logic;
		Signal_i: in std_logic;
		Pulse_o: out std_logic
	);
end entity;

architecture base of PulseGenerator is

	signal ShiftRegister_b: std_logic_vector(g_PulseMinWidth-1 downto 0) := (others => '0');

begin

	pShiftRegister: process (Clk_ik) begin
		if rising_edge(Clk_ik) then
			if Reset_ir = '1' then
				ShiftRegister_b <= (others => '0');
			else
				ShiftRegister_b <= ShiftRegister_b(g_PulseMinWidth-2 downto 0)&Signal_i;
			end if;
		end if;
	end process;
	
	pPulseGenerator: process (ShiftRegister_b) is
		variable Pulse: std_logic := '0';
	begin
		Pulse := '0';
		for i in ShiftRegister_b'range loop
			Pulse := Pulse or ShiftRegister_b(i);
		end loop;
		Pulse_o <= Pulse;
	end process;

end architecture;
