library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity PulseGeneratorTime is
    generic (
        g_ClkFrequency: positive;    -- input clock frequency in Hz
        g_PulseMinWidthInTime: time  -- minimal pulse width
    );
    port (
        Clk_ik: in std_logic;
        Reset_ir: in std_logic;
        Signal_i: in std_logic;
        Pulse_o: out std_logic
    );
end entity;

architecture syn of PulseGeneratorTime is

    constant c_ClkPeriod: time := (1.0 sec)/real(g_ClkFrequency);

    function f_CalculatePulseClk (ClkPeriod, PulseMinWidthInTime: time) return integer is
        variable Result: integer;
    begin
        Result := PulseMinWidthInTime / ClkPeriod;
        if Result < 1 then
            Result := 1;
        end if;
        return Result;
    end function;

    constant c_PulseMinWidth: integer := f_CalculatePulseClk(c_ClkPeriod, g_PulseMinWidthInTime);
    
begin

    cPulseGenerator: entity work.PulseGenerator(base)
        generic map (
            g_PulseMinWidth => c_PulseMinWidth
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => Reset_ir,
            Signal_i => Signal_i,
            Pulse_o => Pulse_o
        );

end architecture;
