library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;
use work.wishbone_pkg.all;

entity FfpgCoreWrapper is
    generic (
        g_ClkFrequency: natural -- input clock frequency in MHz
    );
    port (
        -- Wishbone connection
        clk_sys_i: in std_logic;
        rst_i: in std_logic;
        wb_adr_i: in std_logic_vector(31 downto 0);
        wb_dat_i: in std_logic_vector(31 downto 0);
        wb_dat_o: out std_logic_vector(31 downto 0);
        wb_cyc_i: in std_logic;
        wb_sel_i: in std_logic_vector(3 downto 0);
        wb_stb_i: in std_logic;
        wb_we_i: in std_logic;
        wb_ack_o: out std_logic;
        wb_err_o: out std_logic;
        wb_rty_o: out std_logic;
        wb_stall_o: out std_logic;
        wb_int_o: out std_logic;
        ClkIn0_ik: in std_logic;
        TriggerDacFrameSynchronization_n: out std_logic;
        TriggerDacSerialClock: out std_logic;
        TriggerDacSerialData: out std_logic;
        VcxoDacFrameSynchronization_n: out std_logic;
        VcxoDacSerialClock: out std_logic;
        VcxoDacSerialData: out std_logic;
        Ch1OutputEnable_o: out std_logic;
        Ch2OutputEnable_o: out std_logic;
        DelayValue_ob: out unsigned(9 downto 0);
        Ch1SetLe_o: out std_logic;
        Ch1ResLe_o: out std_logic;
        Ch2SetLe_o: out std_logic;
        Ch2ResLe_o: out std_logic;
        Ch1Set_o: out std_logic;
        Ch1Res_o: out std_logic;
        Ch2Set_o: out std_logic;
        Ch2Res_o: out std_logic;
        Trigger_i: in std_logic;
        SpiAd9512Sclk_o: out std_logic;
        SpiAd9512Mosi_o: out std_logic;
        SpiAd9512Miso_i: in std_logic;
        SpiAd9512Cs_on: out std_logic;
        Clk2Sel_o: out std_logic;
        Onewire_io: inout std_logic;
        Led_ob: out std_logic_vector(4 downto 1)
    );
end entity;

architecture syn of FfpgCoreWrapper is

    signal TriggerDac: t_Ad5600Interface;
    signal VcxoDac: t_Ad5600Interface;
    signal WbMosi: t_wishbone_slave_in;
    signal WbMiso: t_wishbone_slave_out;
    
begin

    WbMosi.adr <= wb_adr_i;
    WbMosi.dat <= wb_dat_i;
    WbMosi.cyc <= wb_cyc_i;
    WbMosi.sel <= wb_sel_i;
    WbMosi.stb <= wb_stb_i;
    WbMosi.we <= wb_we_i;

    cFfpgCore: entity work.FfpgCore(syn)
        generic map (
            g_ClkFrequency => g_ClkFrequency
        )
        port map (
            Clk_ik => clk_sys_i,
            Reset_ira => rst_i,
            Wb_i => WbMosi,
            Wb_o => WbMiso,
            ClkIn0_ik => ClkIn0_ik,
            TriggerDac_o => TriggerDac,
            VcxoDac_o => VcxoDac,
            Ch1OutputEnable_o => Ch1OutputEnable_o,
            Ch2OutputEnable_o => Ch2OutputEnable_o,
            DelayValue_ob => DelayValue_ob,
            Ch1SetLe_o => Ch1SetLe_o,
            Ch1ResLe_o => Ch1ResLe_o,
            Ch2SetLe_o => Ch2SetLe_o,
            Ch2ResLe_o => Ch2ResLe_o,
            Ch1Set_o => Ch1Set_o,
            Ch1Res_o => Ch1Res_o,
            Ch2Set_o => Ch2Set_o,
            Ch2Res_o => Ch2Res_o,
            Trigger_i => Trigger_i,
            SpiAd9512Sclk_o => SpiAd9512Sclk_o,
            SpiAd9512Mosi_o => SpiAd9512Mosi_o,
            SpiAd9512Miso_i => SpiAd9512Miso_i,
            SpiAd9512Cs_on => SpiAd9512Cs_on,
            Clk2Sel_o => Clk2Sel_o,
            Onewire_io => Onewire_io,
            Led_ob => Led_ob
        );

    wb_dat_o <= WbMiso.dat;
    wb_ack_o <= WbMiso.ack;
    wb_err_o <= WbMiso.err;
    wb_rty_o <= WbMiso.rty;
    wb_stall_o <= WbMiso.stall;
    wb_int_o <= WbMiso.int;

    TriggerDacFrameSynchronization_n <= TriggerDac.FrameSynchronization_n;
    TriggerDacSerialClock <= TriggerDac.SerialClock;
    TriggerDacSerialData <= TriggerDac.SerialData;

    VcxoDacFrameSynchronization_n <= VcxoDac.FrameSynchronization_n;
    VcxoDacSerialClock <= VcxoDac.SerialClock;
    VcxoDacSerialData <= VcxoDac.SerialData;

end architecture;
