`include "timescale.svh"
`include "Config.svh"

import uvm_pkg::*;
import Ad5600Package::*;
import TestbenchPackage::*;

module Testbench;

    parameter c_ClkPeriod = 1e9/`CLK_FREQ; // in [ns]
    
    bit Clk_k, Reset_r;

    FfpgInterface LocalInterface(Clk_k, Reset_r);

    FfpgCoreWrapper #(
        .g_ClkFrequency(`CLK_FREQ)
    ) dut (
        .rst_i(Reset_r),
        .clk_sys_i(Clk_k),
        .wb_adr_i(LocalInterface.Wb.adr),
        .wb_dat_i(LocalInterface.Wb.dat_o),
        .wb_dat_o(LocalInterface.Wb.dat_i),
        .wb_cyc_i(LocalInterface.Wb.cyc),
        .wb_sel_i(LocalInterface.Wb.sel),
        .wb_stb_i(LocalInterface.Wb.stb),
        .wb_we_i(LocalInterface.Wb.we),
        .wb_ack_o(LocalInterface.Wb.ack),
        .wb_err_o(LocalInterface.Wb.err),
        .wb_rty_o(LocalInterface.Wb.rty),
        .wb_stall_o(), // doesn't exist in B3 version of WB
        .wb_int_o(), // doesn't exist in B3 version of WB
        .ClkIn0_ik(LocalInterface.Fmc.ClkIn0_k),
        .TriggerDacFrameSynchronization_n(LocalInterface.TriggerDac.FrameSynchronization_n),
        .TriggerDacSerialClock(LocalInterface.TriggerDac.SerialClock),
        .TriggerDacSerialData(LocalInterface.TriggerDac.SerialData),
        .VcxoDacFrameSynchronization_n(LocalInterface.VcxoDac.FrameSynchronization_n),
        .VcxoDacSerialClock(LocalInterface.VcxoDac.SerialClock),
        .VcxoDacSerialData(LocalInterface.VcxoDac.SerialData),
        .Ch1OutputEnable_o(), // not tested so far
        .Ch2OutputEnable_o(), // not tested so far
        .DelayValue_ob(), // not tested so far
        .Ch1SetLe_o(), // not tested so far
        .Ch1ResLe_o(), // not tested so far
        .Ch2SetLe_o(), // not tested so far
        .Ch2ResLe_o(), // not tested so far
        .Ch1Set_o(LocalInterface.Fmc.Ch1Set),
        .Ch1Res_o(LocalInterface.Fmc.Ch1Res),
        .Ch2Set_o(LocalInterface.Fmc.Ch2Set),
        .Ch2Res_o(LocalInterface.Fmc.Ch2Res),
        .Trigger_i(LocalInterface.Fmc.Trigger),
        .SpiAd9512Sclk_o(), // not tested so far
        .SpiAd9512Mosi_o(), // not tested so far
        .SpiAd9512Miso_i(), // not tested so far
        .SpiAd9512Cs_on(), // not tested so far
        .Clk2Sel_o(), // not tested so far
        .Onewire_io(), // not tested so far
        .Led_ob() // not tested so far
    );
    
    initial begin: clockGenerator
        Clk_k = 0;
        forever #(c_ClkPeriod/2) Clk_k = ~Clk_k;
    end

    initial begin: resetGenerator
        Reset_r = 1;
        repeat(3) @(negedge Clk_k);
        Reset_r = 0;
    end
    assign LocalInterface.Wb.rst_i = Reset_r;

    initial begin: mainTest
        // uvm_config_db_options::turn_on_tracing();

        uvm_top.set_report_verbosity_level_hier(UVM_MEDIUM);
        // uvm_top.set_report_verbosity_level_hier(UVM_HIGH);
        // uvm_top.set_report_severity_action_hier(UVM_INFO, UVM_NO_ACTION);
        // uvm_top.set_report_id_action_hier("ja", UVM_NO_ACTION);

        uvm_config_db #(virtual FfpgInterface)::set(null, "uvm_test_top.*", "dut_vi", LocalInterface);
        uvm_top.finish_on_completion = 0; // not to finish when GUI is used

        // run_test("TestTriggerAdc");
        // run_test("TestDelayConfiguration");
        // run_test("TestPulseGenerationBlind");
        // run_test("TestPulseGenerationSequential");
        run_test("TestPulseGeneration");

        $stop();
    end
    
    initial begin: timeout
        #100ms $error("Simulation timeout!");
        $stop();
    end

endmodule