library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity HeartBeat_tb is
end entity;

architecture testbench of HeartBeat_tb is

    constant c_ClkFrequency: positive := 300_000;
    constant c_ClkPeriod: time := (1 sec)/real(c_ClkFrequency);
    
    procedure f_Tick(ticks: in natural) is begin
        wait for ticks * c_ClkPeriod;
    end procedure;

    signal Clk_ik, Reset_ir, HeartBeat_o: std_logic;
    signal HeartBeatPwmValue: time := 0 ns;

begin

    cDUT: entity work.HeartBeat(syn)
        generic map (
            g_ClkFrequency => c_ClkFrequency
        )
        port map (
            Clk_ik,
            Reset_ir,
            HeartBeat_o
        );

    pClk: process is begin
        Clk_ik <= '0';
        wait for c_ClkPeriod/2;
        Clk_ik <= '1';
        wait for c_ClkPeriod/2;
    end process;
    
    pTest: process is begin
        Reset_ir <= '1';
        f_Tick(5);
        Reset_ir <= '0';
        f_Tick(1e6);

        assert false report "NONE. End of simulation." severity failure;
        wait;
    end process pTest;
    
    pRC: process (HeartBeat_o) is
        variable LastOn: time := 0 ns;
    begin
        if rising_edge(HeartBeat_o) then
            LastOn := now;
        else
            HeartBeatPwmValue <= now - LastOn;
        end if;
    end process;

end architecture;
