library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.FfpgPkg.all;

entity SvecTopFfpg is
    port (
        -- Local 20MHz VCXO oscillator
        Clk20_ik: in std_logic;

        -- Reset from system FPGA
        Reset_inr: in std_logic;

        -- DAC interface (20MHz and 25MHz VCXO)
        Pll20DacDin_o: out std_logic;
        Pll20DacSclk_o: out std_logic;
        Pll20DacSync_on: out std_logic;
        Pll25DacDin_o: out std_logic;
        Pll25DacSclk_o: out std_logic;
        Pll25DacSync_on: out std_logic;

        -- Carrier font panel LEDs
        FpLedsLineEnable_ob2: out std_logic_vector(1 downto 0);
        FpLedsLine_ob2: out std_logic_vector(1 downto 0);
        FpLedsColumn_ob4: out std_logic_vector(3 downto 0);

        -- Carrier I2C EEPROM
        CarrierScl_io: inout std_logic;
        CarrierSda_io: inout std_logic;

        -- PCB revision
        PcbRev_i: in std_logic_vector(4 downto 0);

        -- Carrier 1-wire interface (DS18B20 thermometer + unique ID)
        CarrierOneWire_io: inout std_logic;

        ------------------------------------------
        -- VME interface
        ------------------------------------------
        VmeWrite_n_i    : in    std_logic;
        VmeSysreset_n_i : in    std_logic;
        --VmeSysclk_i     : in    std_logic;
        VmeRetry_oe_o   : out   std_logic;
        VmeRetry_n_o    : out   std_logic;
        VmeLword_n_b    : inout std_logic;
        VmeIackout_n_o  : out   std_logic;
        VmeIackin_n_i   : in    std_logic;
        VmeIack_n_i     : in    std_logic;
        --VmeGap_i        : in    std_logic;
        VmeDtack_oe_o   : out   std_logic;
        VmeDtack_n_o    : out   std_logic;
        VmeDs_n_i       : in    std_logic_vector(1 downto 0);
        VmeData_oe_n_o  : out   std_logic;
        VmeData_dir_o   : out   std_logic;
        VmeBerr_o       : out   std_logic;
        VmeAs_n_i       : in    std_logic;
        VmeAddr_oe_n_o  : out   std_logic;
        VmeAddr_dir_o   : out   std_logic;
        VmeIrq_n_o      : out   std_logic_vector(7 downto 1);
        VmeGa_i         : in    std_logic_vector(5 downto 0);
        VmeData_b       : inout std_logic_vector(31 downto 0);
        VmeAm_i         : in    std_logic_vector(5 downto 0);
        VmeAddr_b       : inout std_logic_vector(31 downto 1);

        ------------------------------------------
        -- FMC slot management
        ------------------------------------------
        Fmc0Present_in: in std_logic;   -- Mezzanine present (active low)
        -- Fmc0Scl: inout std_logic;       -- Mezzanine system I2C clock (EEPROM)
        -- Fmc0Sda: inout std_logic;       -- Mezzanine system I2C data (EEPROM)

        Fmc1Present_in: in std_logic;   -- Mezzanine present (active low)
        -- Fmc1Scl: inout std_logic;       -- Mezzanine system I2C clock (EEPROM)
        -- Fmc1Sda: inout std_logic;       -- Mezzanine system I2C data (EEPROM)

        ------------------------------------------
        -- FMC slot 0
        ------------------------------------------

        -- clock
        Fmc0ClkIn0P_ik: in std_logic;
        Fmc0ClkIn0N_ik: in std_logic;
        -- DACs
        Fmc0TriggerDac_o: out t_Ad5600Interface;
        Fmc0VcxoDac_o: out t_Ad5600Interface;
        -- output enable
        Fmc0Ch1OutputEnable_o: out std_logic;
        Fmc0Ch2OutputEnable_o: out std_logic;
        -- delay configuration
        Fmc0DelayValue_ob: out unsigned(9 downto 0);
        Fmc0Ch1SetLe_o: out std_logic;
        Fmc0Ch1ResLe_o: out std_logic;
        Fmc0Ch2SetLe_o: out std_logic;
        Fmc0Ch2ResLe_o: out std_logic;
        Fmc0Ch1SetP_o: out std_logic;
        Fmc0Ch1SetN_o: out std_logic;
        Fmc0Ch1ResP_o: out std_logic;
        Fmc0Ch1ResN_o: out std_logic;
        Fmc0Ch2SetP_o: out std_logic;
        Fmc0Ch2SetN_o: out std_logic;
        Fmc0Ch2ResP_o: out std_logic;
        Fmc0Ch2ResN_o: out std_logic;
        Fmc0TriggerP_i: in std_logic;
        Fmc0TriggerN_i: in std_logic;
        -- AD9512 SPI
        Fmc0SpiAd9512Sclk_o: out std_logic;
        Fmc0SpiAd9512Mosi_o: out std_logic;
        Fmc0SpiAd9512Miso_i: in std_logic;
        Fmc0SpiAd9512Cs_on: out std_logic;
        -- AD9512 func. pin
        Fmc0Ad9512Func_o: out std_logic;
        -- clock selection pin of SY58017U
        Fmc0Clk2Sel_o: out std_logic;
        -- clock output
        Fmc0ClkOutP_ok: out std_logic;
        Fmc0ClkOutN_ok: out std_logic;
        -- temperature chip interface
        Fmc0Onewire_io: inout std_logic;
        -- LEDs
        Fmc0Led_ob: out std_logic_vector(4 downto 1);
        
        ------------------------------------------
        -- FMC slot 1
        ------------------------------------------

        -- clock
        Fmc1ClkIn0P_ik: in std_logic;
        Fmc1ClkIn0N_ik: in std_logic;
        -- DACs
        Fmc1TriggerDac_o: out t_Ad5600Interface;
        Fmc1VcxoDac_o: out t_Ad5600Interface;
        -- output enable
        Fmc1Ch1OutputEnable_o: out std_logic;
        Fmc1Ch2OutputEnable_o: out std_logic;
        -- delay configuration
        Fmc1DelayValue_ob: out unsigned(9 downto 0);
        Fmc1Ch1SetLe_o: out std_logic;
        Fmc1Ch1ResLe_o: out std_logic;
        Fmc1Ch2SetLe_o: out std_logic;
        Fmc1Ch2ResLe_o: out std_logic;
        Fmc1Ch1SetP_o: out std_logic;
        Fmc1Ch1SetN_o: out std_logic;
        Fmc1Ch1ResP_o: out std_logic;
        Fmc1Ch1ResN_o: out std_logic;
        Fmc1Ch2SetP_o: out std_logic;
        Fmc1Ch2SetN_o: out std_logic;
        Fmc1Ch2ResP_o: out std_logic;
        Fmc1Ch2ResN_o: out std_logic;
        Fmc1TriggerP_i: in std_logic;
        Fmc1TriggerN_i: in std_logic;
        -- AD9512 SPI
        Fmc1SpiAd9512Sclk_o: out std_logic;
        Fmc1SpiAd9512Mosi_o: out std_logic;
        Fmc1SpiAd9512Miso_i: in std_logic;
        Fmc1SpiAd9512Cs_on: out std_logic;
        -- AD9512 func. pin
        Fmc1Ad9512Func_o: out std_logic;
        -- clock selection pin of SY58017U
        Fmc1Clk2Sel_o: out std_logic;
        -- clock output
        Fmc1ClkOutP_ok: out std_logic;
        Fmc1ClkOutN_ok: out std_logic;
        -- temperature chip interface
        Fmc1Onewire_io: inout std_logic;
        -- LEDs
        Fmc1Led_ob: out std_logic_vector(4 downto 1)
        
    );
end entity;


architecture rtl of SvecTopFfpg is

    type t_PllSettings is record
        InputFrequency: integer; -- in Hz
        Divide: integer;
        Mult: integer;
        RefJitter: real;
    end record;

    function f_ComputePllDivider(PllSettings: t_PllSettings; OutputFrequency: integer) return integer is
        variable VcoFrequency, Result: integer;
    begin
        assert PllSettings.InputFrequency * PllSettings.Mult rem PllSettings.Divide = 0
            report "Cannot set the PLL - VCO frequency is not an integer!"
            severity failure;
        VcoFrequency := PllSettings.InputFrequency * PllSettings.Mult / PllSettings.Divide;
        assert VcoFrequency rem OutputFrequency = 0
            report "Cannot set the output frequency to "&integer'image(OutputFrequency)&" Hz - PLL DIVIDE is not an integer!"
            severity failure;
        Result := VcoFrequency / OutputFrequency;
        return Result;
    end function;
    
    constant c_PllSettings: t_PllSettings := (
        InputFrequency => 20_000_000,
        Divide => 1,
        Mult => 50,
        RefJitter => 0.016
    );

    constant c_ClkSysFrequency: positive :=  62_500_000; -- in Hz

    -- SVEC carrier CSR constants
    constant c_CarrierType: std_logic_vector(15 downto 0) := X"0002";

    -- Number of slave port(s) on the wishbone crossbar
    constant c_NumWbMasters: integer := 1;

    -- Number of master port(s) on the wishbone crossbar
    constant c_NumWbSlaves: integer := 5;

    -- Wishbone master(s)
    constant c_MasterVmeId: integer := 0;

    -- Wishbone slave(s)
    constant c_SlaveEepromId: integer := 0;
    constant c_SlaveOnewireId: integer := 1;
    constant c_SlaveCsrId: integer := 2;
    constant c_SlaveFmc0Id: integer := 3;
    constant c_SlaveFmc1Id: integer := 4;

    -- Top module repository url
    constant c_SdbRepoUrl: t_sdb_repo_url := (
        repo_url => "git://ohwr.org/fmc-projects/fmc-del-1ns-2cha.git               ");

    -- sdb header address
    constant c_SdbAddress: t_wishbone_address := x"0000_0000";

    constant c_SvecCsrSdb: t_sdb_device := (
        abi_class     => x"0000",                     -- undocumented device
        abi_ver_major => x"01",
        abi_ver_minor => x"01",
        wbd_endian    => c_sdb_endian_big,
        wbd_width     => x"4",                        -- 32-bit port granularity
        sdb_component => (
            addr_first  => x"0000000000000000",
            addr_last   => x"000000000000001F",
            product     => (
                vendor_id => x"000000000000CE42",         -- CERN
                device_id => x"00006603",
                version   => x"00000001",
                date      => x"20121116",
                name      => "WB-SVEC-CSR        ")));

    -- Wishbone crossbar layout
    constant c_InterconnectLayout: t_sdb_record_array(c_NumWbSlaves-1+1 downto 0) := (
        0 => f_sdb_embed_device(c_xwb_i2c_master_sdb, x"0000_1000"),
        1 => f_sdb_embed_device(c_xwb_onewire_master_sdb, x"0000_1100"),
        2 => f_sdb_embed_device(c_SvecCsrSdb, x"0000_1200"),
        3 => f_sdb_embed_bridge(c_FfpgBridgeSdb, x"0002_0000"),
        4 => f_sdb_embed_bridge(c_FfpgBridgeSdb, x"0006_0000"),
        5 => f_sdb_embed_repo_url(c_SdbRepoUrl)
    );
    -- !! maximum addressable VME space on SVEC card is x"0008_0000" !!

    signal Clk20Buffered_k: std_logic;
    signal PllLocked, PllFeedback: std_logic;
    signal ClkSysUnbuffered_k, ClkSys_k: std_logic;

    signal PowerupResetCounter: unsigned(7 downto 0) := (others => '0');
    signal ResetSys_r: std_logic := '1';
    signal ResetSys_nr: std_logic;
    signal SwResetFmc0_r: std_logic := '0';
    signal SwResetFmc0Out_n, SwResetFmc0In_n, SwResetFmc0Load: std_logic;
    signal Fmc0Reset_r: std_logic;
    signal SwResetFmc1_r: std_logic := '0';
    signal SwResetFmc1Out_n, SwResetFmc1In_n, SwResetFmc1Load: std_logic;
    signal Fmc1Reset_r: std_logic;

    signal VmeAddr_b_out: std_logic_vector(31 downto 1);
    signal VmeAddr_dir_int: std_logic;
    signal VmeData_b_out: std_logic_vector(31 downto 0);
    signal VmeData_dir_int: std_logic;
    signal VmeLword_n_b_out: std_logic;
    
    -- Wishbone buse(s) from crossbar master port(s)
    signal CnxMasterOut: t_wishbone_master_out_array(c_NumWbSlaves-1 downto 0);
    signal CnxMasterIn: t_wishbone_master_in_array(c_NumWbSlaves-1 downto 0);

    -- Wishbone buse(s) to crossbar slave port(s)
    signal CnxSlaveOut: t_wishbone_slave_out_array(c_NumWbMasters-1 downto 0);
    signal CnxSlaveIn: t_wishbone_slave_in_array(c_NumWbMasters-1 downto 0);

    signal CarrierOwr, CarrierOwrEn: std_logic;
    signal CarrierSclIn, CarrierSclOe_n, CarrierSclOut: std_logic;
    signal CarrierSdaIn, CarrierSdaOe_n, CarrierSdaOut: std_logic;
    
    signal HeartBeat: std_logic;

    signal LedState: std_logic_vector(15 downto 0);
    signal VmeAccess: std_logic;
    
    -- for differential inputs
    signal Fmc0ClkIn0_k, Fmc0Trigger: std_logic;
    signal Fmc1ClkIn0_k, Fmc1Trigger: std_logic;
    
    -- for differentials outputs
    signal Fmc0Ch1Set, Fmc0Ch1Res, Fmc0Ch2Set, Fmc0Ch2Res: std_logic;
    signal Fmc0ClkOutDdr, Fmc0ClkOut_k: std_logic;
    signal Fmc1Ch1Set, Fmc1Ch1Res, Fmc1Ch2Set, Fmc1Ch2Res: std_logic;
    signal Fmc1ClkOutDdr, Fmc1ClkOut_k: std_logic;

begin

    ------------------------------------------------------------------------------
    -- Clock distribution
    ------------------------------------------------------------------------------

    -- AD5662BRMZ-1 DAC output powers up to 0V. The output remains valid until a
    -- write sequence arrives to the DAC.
    -- To avoid spurious writes, the DAC interface outputs are fixed to safe values.
    Pll20DacDin_o <= '0';
    Pll20DacSclk_o <= '0';
    Pll20DacSync_on <= '1';
    Pll25DacDin_o <= '0';
    Pll25DacSclk_o <= '0';
    Pll25DacSync_on <= '1';

    cClk20Buf: IBUFG
        port map (
            I => Clk20_ik,
            O => Clk20Buffered_k
        );

    cPll: PLL_BASE
        generic map (
            BANDWIDTH => "OPTIMIZED",
            CLK_FEEDBACK => "CLKFBOUT",
            COMPENSATION => "INTERNAL",
            DIVCLK_DIVIDE => c_PllSettings.Divide,
            CLKFBOUT_MULT => c_PllSettings.Mult,
            CLKFBOUT_PHASE => 0.000,
            CLKOUT0_DIVIDE => f_ComputePllDivider(c_PllSettings, c_ClkSysFrequency),
            CLKOUT0_PHASE => 0.000,
            CLKOUT0_DUTY_CYCLE => 0.500,
            CLKIN_PERIOD => (1.0)/real(c_PllSettings.InputFrequency)*1.0e9, -- in ns
            REF_JITTER => c_PllSettings.RefJitter
        )
        port map (
            CLKFBOUT => PllFeedback,
            CLKOUT0 => ClkSysUnbuffered_k,
            CLKOUT1 => open,
            CLKOUT2 => open,
            CLKOUT3 => open,
            CLKOUT4 => open,
            CLKOUT5 => open,
            LOCKED => PllLocked,
            RST => '0',
            CLKFBIN => PllFeedback,
            CLKIN => Clk20Buffered_k
        );

    cClkSysBuf: BUFG
        port map (
            I => ClkSysUnbuffered_k,
            O => ClkSys_k
        );

    ------------------------------------------------------------------------------
    -- System reset
    ------------------------------------------------------------------------------

    pPowerupReset: process (ClkSys_k) begin
        if rising_edge(ClkSys_k) then
            if (VmeSysreset_n_i = '0' or Reset_inr = '0') then
                ResetSys_r <= '1';
            elsif PllLocked = '1' then
                if PowerupResetCounter = "11111111" then
                    ResetSys_r <= '0';
                else
                    ResetSys_r <= '1';
                    PowerupResetCounter <= PowerupResetCounter + 1;
                end if;
            else
                ResetSys_r <= '1';
                PowerupResetCounter <= (others => '0');
            end if;
        end if;
    end process;
    ResetSys_nr <= not ResetSys_r;

    ------------------------------------------------------------------------------
    -- VME interface
    ------------------------------------------------------------------------------

    cVmeCore: entity work.xvme64x_core(wrapper)
        port map (
            clk_i => ClkSys_k,
            rst_n_i => ResetSys_nr,
            VME_AS_n_i => VmeAs_n_i,
            VME_RST_n_i => VmeSysreset_n_i,
            VME_WRITE_n_i => VmeWrite_n_i,
            VME_AM_i => VmeAm_i,
            VME_DS_n_i => VmeDs_n_i,
            VME_GA_i => VmeGa_i,
            VME_BERR_o => VmeBerr_o,
            VME_DTACK_n_o => VmeDtack_n_o,
            VME_RETRY_n_o => VmeRetry_n_o,
            VME_RETRY_OE_o => VmeRetry_oe_o,
            VME_LWORD_n_b_i => VmeLword_n_b,
            VME_LWORD_n_b_o => VmeLword_n_b_out,
            VME_ADDR_b_i => VmeAddr_b,
            VME_DATA_b_o => VmeData_b_out,
            VME_ADDR_b_o => VmeAddr_b_out,
            VME_DATA_b_i => VmeData_b,
            VME_IRQ_n_o => VmeIrq_n_o,
            VME_IACK_n_i => VmeIack_n_i,
            VME_IACKIN_n_i => VmeIackin_n_i,
            VME_IACKOUT_n_o => VmeIackout_n_o,
            VME_DTACK_OE_o => VmeDtack_oe_o,
            VME_DATA_DIR_o => VmeData_dir_int,
            VME_DATA_OE_N_o => VmeData_oe_n_o,
            VME_ADDR_DIR_o => VmeAddr_dir_int,
            VME_ADDR_OE_N_o => VmeAddr_oe_n_o,
            master_o => CnxSlaveIn(c_MasterVmeId),
            master_i => CnxSlaveOut(c_MasterVmeId),
            irq_i => '0'
        );

    -- VME tri-state buffers
    VmeData_b <=
        VmeData_b_out when VmeData_dir_int = '1' else
        (others => 'Z');
    VmeAddr_b <=
        VmeAddr_b_out when VmeAddr_dir_int = '1' else
        (others => 'Z');
    VmeLword_n_b <=
        VmeLword_n_b_out when VmeAddr_dir_int = '1' else
        'Z';

    VmeAddr_dir_o <= VmeAddr_dir_int;
    VmeData_dir_o <= VmeData_dir_int;

    ------------------------------------------------------------------------------
    -- CSR wishbone crossbar
    ------------------------------------------------------------------------------

    cWbSdbCrossbar: entity work.xwb_sdb_crossbar(rtl)
        generic map (
            g_num_masters => c_NumWbMasters,
            g_num_slaves => c_NumWbSlaves,
            g_registered => true,
            g_wraparound => true,
            g_layout => c_InterconnectLayout,
            g_sdb_addr => c_SdbAddress
        )
        port map (
            clk_sys_i => ClkSys_k,
            rst_n_i => ResetSys_nr,
            slave_i => CnxSlaveIn,
            slave_o => CnxSlaveOut,
            master_i => CnxMasterIn,
            master_o => CnxMasterOut
        );

    ------------------------------------------------------------------------------
    -- I2C master
    --    Carrier EEPROM
    ------------------------------------------------------------------------------

    cCarrierI2c: entity work.xwb_i2c_master(rtl)
        generic map (
            g_interface_mode => CLASSIC,
            g_address_granularity => BYTE,
            g_num_interfaces => 1
        )
        port map (
            clk_sys_i => ClkSys_k,
            rst_n_i => ResetSys_nr,
            slave_i => CnxMasterOut(c_SlaveEepromId),
            slave_o => CnxMasterIn(c_SlaveEepromId),
            desc_o => open,
            scl_pad_i(0) => CarrierSclIn,
            scl_pad_o(0) => CarrierSclOut,
            scl_padoen_o(0) => CarrierSclOe_n,
            sda_pad_i(0) => CarrierSdaIn,
            sda_pad_o(0) => CarrierSdaOut,
            sda_padoen_o(0) => CarrierSdaOe_n
        );

    -- Tri-state buffer for SDA and SCL
    CarrierScl_io <=
        CarrierSclOut when CarrierSclOe_n = '0' else
        'Z';
    CarrierSclIn <= CarrierScl_io;

    CarrierSda_io <=
        CarrierSdaOut when CarrierSdaOe_n = '0' else
        'Z';
    CarrierSdaIn <= CarrierSda_io;

    ------------------------------------------------------------------------------
    -- Carrier 1-wire master
    --    DS18B20 (thermometer + unique ID)
    ------------------------------------------------------------------------------

    cCarrierOnewire: entity work.xwb_onewire_master(rtl)
        generic map (
            g_interface_mode => CLASSIC,
            g_address_granularity => BYTE,
            g_num_ports => 1,
            g_ow_btp_normal => "5.0",
            g_ow_btp_overdrive => "1.0",
            g_CDR_N => (c_ClkSysFrequency/1e6)*5 - 1,
            g_CDR_O => (c_ClkSysFrequency/1e6)*1 - 1
        )
        port map (
            clk_sys_i => ClkSys_k,
            rst_n_i => ResetSys_nr,
            slave_i => CnxMasterOut(c_SlaveOnewireId),
            slave_o => CnxMasterIn(c_SlaveOnewireId),
            desc_o => open,
            owr_pwren_o => open,
            owr_en_o(0) => CarrierOwrEn,
            owr_i(0) => CarrierOwr
        );

    CarrierOneWire_io <=
        '0' when CarrierOwrEn = '1' else
        'Z';
    CarrierOwr <= CarrierOneWire_io;

    ------------------------------------------------------------------------------
    -- Carrier CSR
    --    Carrier type and PCB version
    --    Carrier status (PLL, FMC presence)
    ------------------------------------------------------------------------------

    cCarrierCsr: entity work.carrier_csr(syn)
        port map (
            rst_n_i                          => ResetSys_nr,
            clk_sys_i                        => ClkSys_k,
            wb_adr_i                         => CnxMasterOut(c_SlaveCsrId).adr(3 downto 2),  -- CnxMasterOut.adr is byte address
            wb_dat_i                         => CnxMasterOut(c_SlaveCsrId).dat,
            wb_dat_o                         => CnxMasterIn(c_SlaveCsrId).dat,
            wb_cyc_i                         => CnxMasterOut(c_SlaveCsrId).cyc,
            wb_sel_i                         => CnxMasterOut(c_SlaveCsrId).sel,
            wb_stb_i                         => CnxMasterOut(c_SlaveCsrId).stb,
            wb_we_i                          => CnxMasterOut(c_SlaveCsrId).we,
            wb_ack_o                         => CnxMasterIn(c_SlaveCsrId).ack,
            wb_stall_o                       => CnxMasterIn(c_SlaveCsrId).stall,
            carrier_csr_carrier_pcb_rev_i    => PcbRev_i,
            carrier_csr_carrier_reserved_i   => (others => '0'),
            carrier_csr_carrier_type_i       => c_CarrierType,
            carrier_csr_stat_fmc0_pres_i     => Fmc0Present_in,
            carrier_csr_stat_fmc1_pres_i     => Fmc1Present_in,
            carrier_csr_stat_sys_pll_lck_i   => PllLocked,
            carrier_csr_stat_ddr0_cal_done_i => '0', -- not implemented
            carrier_csr_stat_ddr1_cal_done_i => '0', -- not implemented
            carrier_csr_ctrl_fp_leds_man_o   => open, -- not implemented
            carrier_csr_rst_fmc0_n_o         => SwResetFmc0Out_n,
            carrier_csr_rst_fmc0_n_i         => SwResetFmc0In_n,
            carrier_csr_rst_fmc0_n_load_o    => SwResetFmc0Load,
            carrier_csr_rst_fmc1_n_o         => SwResetFmc1Out_n,
            carrier_csr_rst_fmc1_n_i         => SwResetFmc1In_n,
            carrier_csr_rst_fmc1_n_load_o    => SwResetFmc1Load
        );

    -- Unused wishbone signals
    CnxMasterIn(c_SlaveCsrId).err <= '0';
    CnxMasterIn(c_SlaveCsrId).rty <= '0';
    CnxMasterIn(c_SlaveCsrId).int <= '0';

    -- external software reset registers (to assign a non-zero default value)
    pSwResetFmc0: process (ClkSys_k) begin
        if rising_edge(ClkSys_k) then
            if ResetSys_r = '1' then
                SwResetFmc0_r <= '0';
            elsif SwResetFmc0Load = '1' then
                SwResetFmc0_r <= not SwResetFmc0Out_n;
            end if;
        end if;
    end process;

    SwResetFmc0In_n <= not SwResetFmc0_r;
    Fmc0Reset_r <= ResetSys_r or SwResetFmc0_r;

    pSwResetFmc1: process (ClkSys_k) begin
        if rising_edge(ClkSys_k) then
            if ResetSys_r = '1' then
                SwResetFmc1_r <= '0';
            elsif SwResetFmc1Load = '1' then
                SwResetFmc1_r <= not SwResetFmc1Out_n;
            end if;
        end if;
    end process;

    SwResetFmc1In_n <= not SwResetFmc1_r;
    Fmc1Reset_r <= ResetSys_r or SwResetFmc1_r;

    ------------------------------------------------------------------------------
    -- FPGA loaded led (heart beat)
    ------------------------------------------------------------------------------

    cHeartBeat: entity work.HeartBeat(syn)
        generic map (
            g_ClkFrequency => c_ClkSysFrequency
        )
        port map (
            Clk_ik => ClkSys_k,
            Reset_ir => ResetSys_r,
            HeartBeat_o => HeartBeat
        );
    
    ------------------------------------------------------------------------------
    -- Front panel LED control
    ------------------------------------------------------------------------------

    cLedController: entity work.gc_bicolor_led_ctrl
        generic map (
            g_nb_column => 4,
            g_nb_line => 2,
            g_clk_freq => c_ClkSysFrequency,        -- in Hz
            g_refresh_rate => 250                   -- in Hz
        )
        port map (
            clk_i => ClkSys_k,
            rst_n_i => ResetSys_nr,
            led_intensity_i => "1100100",           -- in %
            led_state_i => LedState,
            column_o => FpLedsColumn_ob4,
            line_o => FpLedsLine_ob2,
            line_oen_o => FpLedsLineEnable_ob2
        );
    
    cSlowToggle: entity work.SlowToggle(behavioral)
        generic map (
            g_Width => 1,
            g_Ticks => c_ClkSysFrequency/15
        )
        port map (
            Clk_ik => ClkSys_k,
            Reset_ir => ResetSys_r,
            Test_i => '0',
            Signal_ib(0) => CnxSlaveIn(c_MasterVmeId).cyc,
            Signal_ob(0) => VmeAccess
        );
        
    -- LED 1 : VME access
    LedState(1 downto 0) <=
        c_led_green when VmeAccess = '1' else
        c_led_off;
    
    -- LED 2 :
    LedState(3 downto 2) <= c_led_red;
    
    -- LED 3 :
    LedState(5 downto 4) <= c_led_red_green;
    
    -- LED 4 :
    LedState(7 downto 6) <= '0' & HeartBeat;
    
    -- LED 5 :
    LedState(9 downto 8) <= c_led_off;
    
    -- LED 6 :
    LedState(11 downto 10) <= c_led_off;
    
    -- LED 7 :
    LedState(13 downto 12) <= c_led_off;
    
    -- LED 8 :
    LedState(15 downto 14) <= c_led_off;
    
    ------------------------------------------------------------------------------
    -- FMC Slot 0 - FFPG
    ------------------------------------------------------------------------------

    cFmc0ClkIn0Ibufgds: IBUFGDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => Fmc0ClkIn0P_ik,
            IB => Fmc0ClkIn0N_ik,
            O => Fmc0ClkIn0_k
        );

    cFmc0TriggerIbufds: IBUFDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => Fmc0TriggerP_i,
            IB => Fmc0TriggerN_i,
            O => Fmc0Trigger
        );

    cFmc0FfpgCore: entity work.FfpgCore(syn)
        generic map (
            g_ClkFrequency => c_ClkSysFrequency -- input clock frequency in Hz
        )
        port map (
            -- Wishbone connection
            Clk_ik => ClkSys_k,
            Reset_ira => Fmc0Reset_r, -- synchronized locally for better timing
            Wb_i => CnxMasterOut(c_SlaveFmc0Id),
            Wb_o => CnxMasterIn(c_SlaveFmc0Id),
            ---- FMC interface
            -- clock
            ClkIn0_ik => Fmc0ClkIn0_k,
            -- DACs
            TriggerDac_o => Fmc0TriggerDac_o,
            VcxoDac_o => Fmc0VcxoDac_o,
            -- output enable
            Ch1OutputEnable_o => Fmc0Ch1OutputEnable_o,
            Ch2OutputEnable_o => Fmc0Ch2OutputEnable_o,
            -- delay configuration
            DelayValue_ob => Fmc0DelayValue_ob,
            Ch1SetLe_o => Fmc0Ch1SetLe_o,
            Ch1ResLe_o => Fmc0Ch1ResLe_o,
            Ch2SetLe_o => Fmc0Ch2SetLe_o,
            Ch2ResLe_o => Fmc0Ch2ResLe_o,
            Ch1Set_o => Fmc0Ch1Set,
            Ch1Res_o => Fmc0Ch1Res,
            Ch2Set_o => Fmc0Ch2Set,
            Ch2Res_o => Fmc0Ch2Res,
            Trigger_i => Fmc0Trigger,
            -- AD9512 SPI
            SpiAd9512Sclk_o => Fmc0SpiAd9512Sclk_o,
            SpiAd9512Mosi_o => Fmc0SpiAd9512Mosi_o,
            SpiAd9512Miso_i => Fmc0SpiAd9512Miso_i,
            SpiAd9512Cs_on => Fmc0SpiAd9512Cs_on,
            -- AD9512 func. pin
            Ad9512Func_o => Fmc0Ad9512Func_o,
            -- clock selection pin of SY58017U
            Clk2Sel_o => Fmc0Clk2Sel_o,
            -- clock output
            ClkOut_ok => Fmc0ClkOut_k,
            -- temperature chip interface
            Onewire_io => Fmc0Onewire_io,
            -- LEDs
            Led_ob => Fmc0Led_ob
        );
    
    cFmc0Ch1SetObufds: OBUFDS
        port map (
            I => Fmc0Ch1Set,
            O => Fmc0Ch1SetP_o,
            OB => Fmc0Ch1SetN_o
        );

    cFmc0Ch1ResObufds: OBUFDS
        port map (
            I => Fmc0Ch1Res,
            O => Fmc0Ch1ResP_o,
            OB => Fmc0Ch1ResN_o
        );

    cFmc0Ch2SetObufds: OBUFDS
        port map (
            I => Fmc0Ch2Set,
            O => Fmc0Ch2SetP_o,
            OB => Fmc0Ch2SetN_o
        );

    cFmc0Ch2ResObufds: OBUFDS
        port map (
            I => Fmc0Ch2Res,
            O => Fmc0Ch2ResP_o,
            OB => Fmc0Ch2ResN_o
        );

    cFmc0ClkOutOddr: ODDR2
        generic map(
            DDR_ALIGNMENT => "C0",
            SRTYPE => "ASYNC"
        )
        port map (
            Q => Fmc0ClkOutDdr,
            C0 => Fmc0ClkOut_k,
            C1 => not Fmc0ClkOut_k,
            CE => '1',
            D0 => '1',
            D1 => '0',
            R => '0',
            S => '0'
        );
        
    cFmc0ClkOutObufds: OBUFDS
        port map (
            I => Fmc0ClkOutDdr,
            O => Fmc0ClkOutP_ok,
            OB => Fmc0ClkOutN_ok
        );

    ------------------------------------------------------------------------------
    -- FMC Slot 1
    ------------------------------------------------------------------------------

    cFmc1ClkIn0Ibufgds: IBUFGDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => Fmc1ClkIn0P_ik,
            IB => Fmc1ClkIn0N_ik,
            O => Fmc1ClkIn0_k
        );

    cFmc1TriggerIbufds: IBUFDS
        generic map (
            DIFF_TERM => TRUE
        )
        port map (
            I => Fmc1TriggerP_i,
            IB => Fmc1TriggerN_i,
            O => Fmc1Trigger
        );

    cFmc1FfpgCore: entity work.FfpgCore(syn)
        generic map (
            g_ClkFrequency => c_ClkSysFrequency -- input clock frequency in Hz
        )
        port map (
            -- Wishbone connection
            Clk_ik => ClkSys_k,
            Reset_ira => Fmc1Reset_r, -- synchronized locally for better timing
            Wb_i => CnxMasterOut(c_SlaveFmc1Id),
            Wb_o => CnxMasterIn(c_SlaveFmc1Id),
            ---- FMC interface
            -- clock
            ClkIn0_ik => Fmc1ClkIn0_k,
            -- DACs
            TriggerDac_o => Fmc1TriggerDac_o,
            VcxoDac_o => Fmc1VcxoDac_o,
            -- output enable
            Ch1OutputEnable_o => Fmc1Ch1OutputEnable_o,
            Ch2OutputEnable_o => Fmc1Ch2OutputEnable_o,
            -- delay configuration
            DelayValue_ob => Fmc1DelayValue_ob,
            Ch1SetLe_o => Fmc1Ch1SetLe_o,
            Ch1ResLe_o => Fmc1Ch1ResLe_o,
            Ch2SetLe_o => Fmc1Ch2SetLe_o,
            Ch2ResLe_o => Fmc1Ch2ResLe_o,
            Ch1Set_o => Fmc1Ch1Set,
            Ch1Res_o => Fmc1Ch1Res,
            Ch2Set_o => Fmc1Ch2Set,
            Ch2Res_o => Fmc1Ch2Res,
            Trigger_i => Fmc1Trigger,
            -- AD9512 SPI
            SpiAd9512Sclk_o => Fmc1SpiAd9512Sclk_o,
            SpiAd9512Mosi_o => Fmc1SpiAd9512Mosi_o,
            SpiAd9512Miso_i => Fmc1SpiAd9512Miso_i,
            SpiAd9512Cs_on => Fmc1SpiAd9512Cs_on,
            -- AD9512 func. pin
            Ad9512Func_o => Fmc1Ad9512Func_o,
            -- clock selection pin of SY58017U
            Clk2Sel_o => Fmc1Clk2Sel_o,
            -- clock output
            ClkOut_ok => Fmc1ClkOut_k,
            -- temperature chip interface
            Onewire_io => Fmc1Onewire_io,
            -- LEDs
            Led_ob => Fmc1Led_ob
        );
    
    cFmc1Ch1SetObufds: OBUFDS
        port map (
            I => Fmc1Ch1Set,
            O => Fmc1Ch1SetP_o,
            OB => Fmc1Ch1SetN_o
        );

    cFmc1Ch1ResObufds: OBUFDS
        port map (
            I => Fmc1Ch1Res,
            O => Fmc1Ch1ResP_o,
            OB => Fmc1Ch1ResN_o
        );

    cFmc1Ch2SetObufds: OBUFDS
        port map (
            I => Fmc1Ch2Set,
            O => Fmc1Ch2SetP_o,
            OB => Fmc1Ch2SetN_o
        );

    cFmc1Ch2ResObufds: OBUFDS
        port map (
            I => Fmc1Ch2Res,
            O => Fmc1Ch2ResP_o,
            OB => Fmc1Ch2ResN_o
        );

    cFmc1ClkOutOddr: ODDR2
        generic map(
            DDR_ALIGNMENT => "C0",
            SRTYPE => "ASYNC"
        )
        port map (
            Q => Fmc1ClkOutDdr,
            C0 => Fmc1ClkOut_k,
            C1 => not Fmc1ClkOut_k,
            CE => '1',
            D0 => '1',
            D1 => '0',
            R => '0',
            S => '0'
        );
        
    cFmc1ClkOutObufds: OBUFDS
        port map (
            I => Fmc1ClkOutDdr,
            O => Fmc1ClkOutP_ok,
            OB => Fmc1ClkOutN_ok
        );

end architecture;

-- -- Unused architecure need to be commented out - ISE doesn't know how to handle multiple architecure
-- -- for top level entity!

-- architecture FmcIoTest of SvecTopFfpg is

    -- signal Fmc0ClkIn0_k, Fmc0Trigger: std_logic;
    -- signal HeartBeat, InputOr: std_logic;

-- begin

    -- -- DAC interface (20MHz and 25MHz VCXO)
    -- Pll20DacDin_o <= '0';
    -- Pll20DacSclk_o <= '0';
    -- Pll20DacSync_on <= '1';
    -- Pll25DacDin_o <= '0';
    -- Pll25DacSclk_o <= '0';
    -- Pll25DacSync_on <= '1';

    -- -- Carrier font panel LEDs
    -- FpLedsLineEnable_ob2 <= (others => '0');
    -- FpLedsLine_ob2 <= (others => '0');
    -- FpLedsColumn_ob4 <= (others => '0');

    -- -- Carrier I2C eeprom
    -- CarrierScl_io <= 'Z';
    -- CarrierSda_io <= 'Z';

    -- -- Carrier 1-wire interface (DS18B20 thermometer + unique ID)
    -- CarrierOneWire_io <= 'Z';

    -- ------------------------------------------
    -- -- VME interface
    -- ------------------------------------------
    -- VmeRetry_oe_o <= '0';
    -- VmeRetry_n_o <= '1';
    -- VmeLword_n_b <= 'Z';
    -- VmeIackout_n_o <= '1';
    -- VmeDtack_oe_o <= '0';
    -- VmeDtack_n_o <= '1';
    -- VmeData_oe_n_o <= '1';
    -- VmeData_dir_o <= '0';
    -- VmeBerr_o <= '0';
    -- VmeAddr_oe_n_o <= '1';
    -- VmeAddr_dir_o <= '0';
    -- VmeIrq_n_o <= (others => '1');
    -- VmeData_b <= (others => 'Z');
    -- VmeAddr_b <= (others => 'Z');

    -- ------------------------------------------
    -- -- Test stuff
    -- ------------------------------------------

    -- cHeartBeat: entity work.HeartBeat(syn)
        -- generic map (
            -- g_ClkFrequency => 20e6
        -- )
        -- port map (
            -- Clk_ik => Clk20_ik,
            -- Reset_ir => not Reset_inr,
            -- HeartBeat_o => HeartBeat
        -- );
    
    -- ------------------------------------------
    -- -- FMC slot 0
    -- ------------------------------------------

    -- -- DACs
    -- Fmc0TriggerDac_o.FrameSynchronization_n <= HeartBeat;
    -- Fmc0TriggerDac_o.SerialClock <= HeartBeat;
    -- Fmc0TriggerDac_o.SerialData <= HeartBeat;
    -- Fmc0VcxoDac_o.FrameSynchronization_n <= HeartBeat;
    -- Fmc0VcxoDac_o.SerialClock <= HeartBeat;
    -- Fmc0VcxoDac_o.SerialData <= HeartBeat;
    -- -- output enable
    -- Fmc0Ch1OutputEnable_o <= HeartBeat;
    -- Fmc0Ch2OutputEnable_o <= HeartBeat;
    -- -- delay configuration
    -- Fmc0DelayValue_ob <= (others => HeartBeat);
    -- Fmc0Ch1SetLe_o <= Fmc0ClkIn0_k;
    -- Fmc0Ch1ResLe_o <= Fmc0Trigger;
    -- Fmc0Ch2SetLe_o <= Fmc0SpiAd9512Miso_i;
    -- Fmc0Ch2ResLe_o <= HeartBeat;
    -- -- AD9512 SPI
    -- Fmc0SpiAd9512Sclk_o <= HeartBeat;
    -- Fmc0SpiAd9512Mosi_o <= HeartBeat;
    -- Fmc0SpiAd9512Cs_on <= HeartBeat;
    -- -- clock selection pin of SY58017U
    -- Fmc0Clk2Sel_o <= HeartBeat;
    -- -- temperature chip interface
    -- Fmc0Onewire_io <= HeartBeat;
    -- -- LEDs
    -- Fmc0Led_ob <= (others => HeartBeat);

    -- -- DS inputs
    
    -- cFmc0ClkIn0Ibufgds: IBUFGDS
        -- generic map (
            -- DIFF_TERM => TRUE
        -- )
        -- port map (
            -- I => Fmc0ClkIn0P_ik,
            -- IB => Fmc0ClkIn0N_ik,
            -- O => Fmc0ClkIn0_k
        -- );

    -- cFmc0TriggerIbufds: IBUFDS
        -- generic map (
            -- DIFF_TERM => TRUE
        -- )
        -- port map (
            -- I => Fmc0TriggerP_i,
            -- IB => Fmc0TriggerN_i,
            -- O => Fmc0Trigger
        -- );

    -- -- DS outputs
    -- cFmc0Ch1SetObufds: OBUFDS
        -- port map (
            -- I => HeartBeat,
            -- O => Fmc0Ch1SetP_o,
            -- OB => Fmc0Ch1SetN_o
        -- );

    -- cFmc0Ch1ResObufds: OBUFDS
        -- port map (
            -- I => HeartBeat,
            -- O => Fmc0Ch1ResP_o,
            -- OB => Fmc0Ch1ResN_o
        -- );

    -- cFmc0Ch2SetObufds: OBUFDS
        -- port map (
            -- I => HeartBeat,
            -- O => Fmc0Ch2SetP_o,
            -- OB => Fmc0Ch2SetN_o
        -- );

    -- cFmc0Ch2ResObufds: OBUFDS
        -- port map (
            -- I => HeartBeat,
            -- O => Fmc0Ch2ResP_o,
            -- OB => Fmc0Ch2ResN_o
        -- );

-- end architecture;