library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;

entity SvecTopFfpgWrapper is
    port (
        -- Local 20MHz VCXO oscillator
        Clk20_ik: in std_logic;

        -- Reset from system fpga
        Reset_inr: in std_logic;

        -- DAC interface (20MHz and 25MHz VCXO)
        Pll20DacDin_o: out std_logic;
        Pll20DacSclk_o: out std_logic;
        Pll20DacSync_on: out std_logic;
        Pll25DacDin_o: out std_logic;
        Pll25DacSclk_o: out std_logic;
        Pll25DacSync_on: out std_logic;

        -- Carrier font panel LEDs
        FpLedsLineEnable_ob2: out std_logic_vector(1 downto 0);
        FpLedsLine_ob2: out std_logic_vector(1 downto 0);
        FpLedsColumn_ob4: out std_logic_vector(3 downto 0);

        -- Carrier I2C eeprom
        CarrierScl_io: inout std_logic;
        CarrierSda_io: inout std_logic;

        -- PCB revision
        PcbRev_i: in std_logic_vector(4 downto 0);

        -- Carrier 1-wire interface (DS18B20 thermometer + unique ID)
        CarrierOneWire_io: inout std_logic;

        ------------------------------------------
        -- VME interface
        ------------------------------------------
        VmeWrite_n_i    : in    std_logic;
        VmeSysreset_n_i : in    std_logic;
        VmeRetry_oe_o   : out   std_logic;
        VmeRetry_n_o    : out   std_logic;
        VmeLword_n_b    : inout std_logic;
        VmeIackout_n_o  : out   std_logic;
        VmeIackin_n_i   : in    std_logic;
        VmeIack_n_i     : in    std_logic;
        VmeDtack_oe_o   : out   std_logic;
        VmeDtack_n_o    : out   std_logic;
        VmeDs_n_i       : in    std_logic_vector(1 downto 0);
        VmeData_oe_n_o  : out   std_logic;
        VmeData_dir_o   : out   std_logic;
        VmeBerr_o       : out   std_logic;
        VmeAs_n_i       : in    std_logic;
        VmeAddr_oe_n_o  : out   std_logic;
        VmeAddr_dir_o   : out   std_logic;
        VmeIrq_n_o      : out   std_logic_vector(7 downto 1);
        VmeGa_i         : in    std_logic_vector(5 downto 0);
        VmeData_b       : inout std_logic_vector(31 downto 0);
        VmeAm_i         : in    std_logic_vector(5 downto 0);
        VmeAddr_b       : inout std_logic_vector(31 downto 1);

        ------------------------------------------
        -- FMC slot management
        ------------------------------------------
        Fmc0Present_in: in std_logic;   -- Mezzanine present (active low)
        -- Fmc0Scl: inout std_logic;       -- Mezzanine system I2C clock (EEPROM)
        -- Fmc0Sda: inout std_logic;       -- Mezzanine system I2C data (EEPROM)

        Fmc1Present_in: in std_logic;   -- Mezzanine present (active low)
        -- Fmc1Scl: inout std_logic;       -- Mezzanine system I2C clock (EEPROM)
        -- Fmc1Sda: inout std_logic;       -- Mezzanine system I2C data (EEPROM)

        ------------------------------------------
        -- FMC slot 0
        ------------------------------------------

        -- clock
        Fmc0ClkIn0P_ik: in std_logic;
        Fmc0ClkIn0N_ik: in std_logic;
        -- DACs
        Fmc0TriggerDac_o_FrameSynchronization_n: out std_logic;
        Fmc0TriggerDac_o_SerialClock: out std_logic;
        Fmc0TriggerDac_o_SerialData: out std_logic;
        Fmc0VcxoDac_o_FrameSynchronization_n: out std_logic;
        Fmc0VcxoDac_o_SerialClock: out std_logic;
        Fmc0VcxoDac_o_SerialData: out std_logic;
        -- output enable
        Fmc0Ch1OutputEnable_o: out std_logic;
        Fmc0Ch2OutputEnable_o: out std_logic;
        -- delay configuration
        Fmc0DelayValue_ob: out std_logic_vector(9 downto 0);
        Fmc0Ch1SetLe_o: out std_logic;
        Fmc0Ch1ResLe_o: out std_logic;
        Fmc0Ch2SetLe_o: out std_logic;
        Fmc0Ch2ResLe_o: out std_logic;
        Fmc0Ch1SetP_o: out std_logic;
        Fmc0Ch1SetN_o: out std_logic;
        Fmc0Ch1ResP_o: out std_logic;
        Fmc0Ch1ResN_o: out std_logic;
        Fmc0Ch2SetP_o: out std_logic;
        Fmc0Ch2SetN_o: out std_logic;
        Fmc0Ch2ResP_o: out std_logic;
        Fmc0Ch2ResN_o: out std_logic;
        Fmc0TriggerP_i: in std_logic;
        Fmc0TriggerN_i: in std_logic;
        -- AD9512 SPI
        Fmc0SpiAd9512Sclk_o: out std_logic;
        Fmc0SpiAd9512Mosi_o: out std_logic;
        Fmc0SpiAd9512Miso_i: in std_logic;
        Fmc0SpiAd9512Cs_on: out std_logic;
        -- AD9512 func. pin
        Fmc0Ad9512Func_o: out std_logic;
        -- clock selection pin of SY58017U
        Fmc0Clk2Sel_o: out std_logic;
        -- clock output
        Fmc0ClkOutP_ok: out std_logic;
        Fmc0ClkOutN_ok: out std_logic;
        -- temperature chip interface
        Fmc0Onewire_io: inout std_logic;
        -- LEDs
        Fmc0Led_ob: out std_logic_vector(4 downto 1);
        
        ------------------------------------------
        -- FMC slot 1
        ------------------------------------------

            -- clock
        Fmc1ClkIn0P_ik: in std_logic;
        Fmc1ClkIn0N_ik: in std_logic;
        -- DACs
        Fmc1TriggerDac_o_FrameSynchronization_n: out std_logic;
        Fmc1TriggerDac_o_SerialClock: out std_logic;
        Fmc1TriggerDac_o_SerialData: out std_logic;
        Fmc1VcxoDac_o_FrameSynchronization_n: out std_logic;
        Fmc1VcxoDac_o_SerialClock: out std_logic;
        Fmc1VcxoDac_o_SerialData: out std_logic;
        -- output enable
        Fmc1Ch1OutputEnable_o: out std_logic;
        Fmc1Ch2OutputEnable_o: out std_logic;
        -- delay configuration
        Fmc1DelayValue_ob: out std_logic_vector(9 downto 0);
        Fmc1Ch1SetLe_o: out std_logic;
        Fmc1Ch1ResLe_o: out std_logic;
        Fmc1Ch2SetLe_o: out std_logic;
        Fmc1Ch2ResLe_o: out std_logic;
        Fmc1Ch1SetP_o: out std_logic;
        Fmc1Ch1SetN_o: out std_logic;
        Fmc1Ch1ResP_o: out std_logic;
        Fmc1Ch1ResN_o: out std_logic;
        Fmc1Ch2SetP_o: out std_logic;
        Fmc1Ch2SetN_o: out std_logic;
        Fmc1Ch2ResP_o: out std_logic;
        Fmc1Ch2ResN_o: out std_logic;
        Fmc1TriggerP_i: in std_logic;
        Fmc1TriggerN_i: in std_logic;
        -- AD9512 SPI
        Fmc1SpiAd9512Sclk_o: out std_logic;
        Fmc1SpiAd9512Mosi_o: out std_logic;
        Fmc1SpiAd9512Miso_i: in std_logic;
        Fmc1SpiAd9512Cs_on: out std_logic;
        -- AD9512 func. pin
        Fmc1Ad9512Func_o: out std_logic;
        -- clock selection pin of SY58017U
        Fmc1Clk2Sel_o: out std_logic;
        -- clock output
        Fmc1ClkOutP_ok: out std_logic;
        Fmc1ClkOutN_ok: out std_logic;
        -- temperature chip interface
        Fmc1Onewire_io: inout std_logic;
        -- LEDs
        Fmc1Led_ob: out std_logic_vector(4 downto 1)

    );
end entity;


architecture wrapper of SvecTopFfpgWrapper is

    signal Fmc0TriggerDac_o, Fmc0VcxoDac_o: t_Ad5600Interface;
    signal Fmc0DelayValue_ob_unsigned: unsigned(9 downto 0);

    signal Fmc1TriggerDac_o, Fmc1VcxoDac_o: t_Ad5600Interface;
    signal Fmc1DelayValue_ob_unsigned: unsigned(9 downto 0);

begin

    cSvecTopFfpg: entity work.SvecTopFfpg(rtl)
        port map (
            Clk20_ik => Clk20_ik,
            Reset_inr => Reset_inr,
            Pll20DacDin_o => Pll20DacDin_o,
            Pll20DacSclk_o => Pll20DacSclk_o,
            Pll20DacSync_on => Pll20DacSync_on,
            Pll25DacDin_o => Pll25DacDin_o,
            Pll25DacSclk_o => Pll25DacSclk_o,
            Pll25DacSync_on => Pll25DacSync_on,
            FpLedsLineEnable_ob2 => FpLedsLineEnable_ob2,
            FpLedsLine_ob2 => FpLedsLine_ob2,
            FpLedsColumn_ob4 => FpLedsColumn_ob4,
            CarrierScl_io => CarrierScl_io,
            CarrierSda_io => CarrierSda_io,
            PcbRev_i => PcbRev_i,
            CarrierOneWire_io => CarrierOneWire_io,
            ------------------------------------------
            -- VME interface
            ------------------------------------------
            VmeWrite_n_i => VmeWrite_n_i,
            VmeSysreset_n_i => VmeSysreset_n_i,
            VmeRetry_oe_o => VmeRetry_oe_o,
            VmeRetry_n_o => VmeRetry_n_o,
            VmeLword_n_b => VmeLword_n_b,
            VmeIackout_n_o => VmeIackout_n_o,
            VmeIackin_n_i => VmeIackin_n_i,
            VmeIack_n_i => VmeIack_n_i,
            VmeDtack_oe_o => VmeDtack_oe_o,
            VmeDtack_n_o => VmeDtack_n_o,
            VmeDs_n_i => VmeDs_n_i,
            VmeData_oe_n_o => VmeData_oe_n_o,
            VmeData_dir_o => VmeData_dir_o,
            VmeBerr_o => VmeBerr_o,
            VmeAs_n_i => VmeAs_n_i,
            VmeAddr_oe_n_o => VmeAddr_oe_n_o,
            VmeAddr_dir_o => VmeAddr_dir_o,
            VmeIrq_n_o => VmeIrq_n_o,
            VmeGa_i => VmeGa_i,
            VmeData_b => VmeData_b,
            VmeAm_i => VmeAm_i,
            VmeAddr_b => VmeAddr_b,
            ------------------------------------------
            -- FMC slot management
            ------------------------------------------
            Fmc0Present_in => Fmc0Present_in,   -- Mezzanine present (active low)
            Fmc1Present_in => Fmc1Present_in,   -- Mezzanine present (active low)
            ------------------------------------------
            -- FMC slot 0
            ------------------------------------------
            Fmc0ClkIn0P_ik => Fmc0ClkIn0P_ik,
            Fmc0ClkIn0N_ik => Fmc0ClkIn0N_ik,
            Fmc0TriggerDac_o.FrameSynchronization_n => Fmc0TriggerDac_o_FrameSynchronization_n,
            Fmc0TriggerDac_o.SerialClock => Fmc0TriggerDac_o_SerialClock,
            Fmc0TriggerDac_o.SerialData => Fmc0TriggerDac_o_SerialData,
            Fmc0VcxoDac_o.FrameSynchronization_n => Fmc0VcxoDac_o_FrameSynchronization_n,
            Fmc0VcxoDac_o.SerialClock => Fmc0VcxoDac_o_SerialClock,
            Fmc0VcxoDac_o.SerialData => Fmc0VcxoDac_o_SerialData,
            Fmc0Ch1OutputEnable_o => Fmc0Ch1OutputEnable_o,
            Fmc0Ch2OutputEnable_o => Fmc0Ch2OutputEnable_o,
            Fmc0DelayValue_ob => Fmc0DelayValue_ob_unsigned,
            Fmc0Ch1SetLe_o => Fmc0Ch1SetLe_o,
            Fmc0Ch1ResLe_o => Fmc0Ch1ResLe_o,
            Fmc0Ch2SetLe_o => Fmc0Ch2SetLe_o,
            Fmc0Ch2ResLe_o => Fmc0Ch2ResLe_o,
            Fmc0Ch1SetP_o => Fmc0Ch1SetP_o,
            Fmc0Ch1SetN_o => Fmc0Ch1SetN_o,
            Fmc0Ch1ResP_o => Fmc0Ch1ResP_o,
            Fmc0Ch1ResN_o => Fmc0Ch1ResN_o,
            Fmc0Ch2SetP_o => Fmc0Ch2SetP_o,
            Fmc0Ch2SetN_o => Fmc0Ch2SetN_o,
            Fmc0Ch2ResP_o => Fmc0Ch2ResP_o,
            Fmc0Ch2ResN_o => Fmc0Ch2ResN_o,
            Fmc0TriggerP_i => Fmc0TriggerP_i,
            Fmc0TriggerN_i => Fmc0TriggerN_i,
            Fmc0SpiAd9512Sclk_o => Fmc0SpiAd9512Sclk_o,
            Fmc0SpiAd9512Mosi_o => Fmc0SpiAd9512Mosi_o,
            Fmc0SpiAd9512Miso_i => Fmc0SpiAd9512Miso_i,
            Fmc0SpiAd9512Cs_on => Fmc0SpiAd9512Cs_on,
            Fmc0Ad9512Func_o => Fmc0Ad9512Func_o,
            Fmc0Clk2Sel_o => Fmc0Clk2Sel_o,
            Fmc0ClkOutP_ok => Fmc0ClkOutP_ok,
            Fmc0ClkOutN_ok => Fmc0ClkOutN_ok,
            Fmc0Onewire_io => Fmc0Onewire_io,
            Fmc0Led_ob => Fmc0Led_ob,
            ------------------------------------------
            -- FMC slot 1
            ------------------------------------------
            Fmc1ClkIn0P_ik => Fmc1ClkIn0P_ik,
            Fmc1ClkIn0N_ik => Fmc1ClkIn0N_ik,
            Fmc1TriggerDac_o.FrameSynchronization_n => Fmc1TriggerDac_o_FrameSynchronization_n,
            Fmc1TriggerDac_o.SerialClock => Fmc1TriggerDac_o_SerialClock,
            Fmc1TriggerDac_o.SerialData => Fmc1TriggerDac_o_SerialData,
            Fmc1VcxoDac_o.FrameSynchronization_n => Fmc1VcxoDac_o_FrameSynchronization_n,
            Fmc1VcxoDac_o.SerialClock => Fmc1VcxoDac_o_SerialClock,
            Fmc1VcxoDac_o.SerialData => Fmc1VcxoDac_o_SerialData,
            Fmc1Ch1OutputEnable_o => Fmc1Ch1OutputEnable_o,
            Fmc1Ch2OutputEnable_o => Fmc1Ch2OutputEnable_o,
            Fmc1DelayValue_ob => Fmc1DelayValue_ob_unsigned,
            Fmc1Ch1SetLe_o => Fmc1Ch1SetLe_o,
            Fmc1Ch1ResLe_o => Fmc1Ch1ResLe_o,
            Fmc1Ch2SetLe_o => Fmc1Ch2SetLe_o,
            Fmc1Ch2ResLe_o => Fmc1Ch2ResLe_o,
            Fmc1Ch1SetP_o => Fmc1Ch1SetP_o,
            Fmc1Ch1SetN_o => Fmc1Ch1SetN_o,
            Fmc1Ch1ResP_o => Fmc1Ch1ResP_o,
            Fmc1Ch1ResN_o => Fmc1Ch1ResN_o,
            Fmc1Ch2SetP_o => Fmc1Ch2SetP_o,
            Fmc1Ch2SetN_o => Fmc1Ch2SetN_o,
            Fmc1Ch2ResP_o => Fmc1Ch2ResP_o,
            Fmc1Ch2ResN_o => Fmc1Ch2ResN_o,
            Fmc1TriggerP_i => Fmc1TriggerP_i,
            Fmc1TriggerN_i => Fmc1TriggerN_i,
            Fmc1SpiAd9512Sclk_o => Fmc1SpiAd9512Sclk_o,
            Fmc1SpiAd9512Mosi_o => Fmc1SpiAd9512Mosi_o,
            Fmc1SpiAd9512Miso_i => Fmc1SpiAd9512Miso_i,
            Fmc1SpiAd9512Cs_on => Fmc1SpiAd9512Cs_on,
            Fmc1Ad9512Func_o => Fmc1Ad9512Func_o,
            Fmc1Clk2Sel_o => Fmc1Clk2Sel_o,
            Fmc1ClkOutP_ok => Fmc1ClkOutP_ok,
            Fmc1ClkOutN_ok => Fmc1ClkOutN_ok,
            Fmc1Onewire_io => Fmc1Onewire_io,
            Fmc1Led_ob => Fmc1Led_ob
        );
        
        Fmc0DelayValue_ob <= std_logic_vector(Fmc0DelayValue_ob_unsigned);
        Fmc1DelayValue_ob <= std_logic_vector(Fmc1DelayValue_ob_unsigned);

end architecture;