`include "vme64x_bfm.svh"
`include "svec_vme_buffers.svh"
`include "fmc.svh"
`include "../../../ffpg/sim/testbench/ffpg_csr.svh"
`include "carrier_csr.svh"
`include "ConfigSvec.svh"

module Testbench;

    reg Reset_inr = 0;
    reg Clk20_ik = 0;
    reg ClkRf_k = 0;
    reg Trigger = 0;
    int BunchCounter = 0;

    always #25ns Clk20_ik <= ~Clk20_ik;
    always #5ns ClkRf_k <= ~ClkRf_k; // 200 MHz
    
    always_ff @(posedge ClkRf_k) begin
        if (BunchCounter == 17820) begin
            BunchCounter <= 0;
            Trigger <= 1;
        end else begin
            BunchCounter <= BunchCounter + 1;
            Trigger <= 0;
        end
    end

    initial begin
        #10us; // for PLL lock and reset settle
        repeat(20) @(posedge Clk20_ik);
            Reset_inr = 1;
    end

    IVME64X VME(Reset_inr);

    `DECLARE_VME_BUFFERS(VME.slave);

    `DECLARE_FMC(0);
    `DECLARE_FMC(1);
    
    logic [1:0] FpLedsLineEnable_ob2;
    logic [1:0] FpLedsLine_ob2;
    logic [3:0] FpLedsColumn_ob4;
    wire        CarrierScl_io;
    wire        CarrierSda_io;
    wire        CarrierOneWire_io;

    pullup(CarrierScl_io);
    pullup(CarrierSda_io);

    CBusAccessor_VME64x acc;

    SvecTopFfpgWrapper cDut (
        .Clk20_ik(Clk20_ik),
        .Reset_inr(Reset_inr),
        .Pll20DacDin_o(),
        .Pll20DacSclk_o(),
        .Pll20DacSync_on(),
        .Pll25DacDin_o(),
        .Pll25DacSclk_o(),
        .Pll25DacSync_on(),
        .FpLedsLineEnable_ob2(FpLedsLineEnable_ob2),
        .FpLedsLine_ob2(FpLedsLine_ob2),
        .FpLedsColumn_ob4(FpLedsColumn_ob4),
        .CarrierScl_io(CarrierScl_io),
        .CarrierSda_io(CarrierSda_io),
        .PcbRev_i(5'b00001),
        .CarrierOneWire_io(CarrierOneWire_io),

        `WIRE_FMC(0)
        `WIRE_FMC(1)
        
        `WIRE_VME_PINS(8) // slot number in parameter // don't change that magic 8 - it's hardcoded somewhere else
    );
    
    assign Fmc0Present_in = 1;
    assign Fmc0ClkIn0P_ik = ClkRf_k;
    assign Fmc0ClkIn0N_ik = ~Fmc0ClkIn0P_ik;
    assign Fmc0TriggerP_i = Trigger;
    assign Fmc0TriggerN_i = ~Fmc0TriggerP_i;
    assign Fmc0SpiAd9512Miso_i = Fmc0SpiAd9512Mosi_o; // loopback

    assign Fmc1Present_in = 1;
    assign Fmc1ClkIn0P_ik = ClkRf_k;
    assign Fmc1ClkIn0N_ik = ~Fmc1ClkIn0P_ik;
    assign Fmc1TriggerP_i = Trigger;
    assign Fmc1TriggerN_i = ~Fmc1TriggerP_i;
    assign Fmc1SpiAd9512Miso_i = Fmc1SpiAd9512Mosi_o; // loopback

    task automatic init_vme64x_core;
        /* map func0 to 0x80000000, A32 */

        acc.write('h7ff63, 'h80, A32|CR_CSR|D08Byte3); // maybe here?
        acc.write('h7ff67, 0, CR_CSR|A32|D08Byte3);
        acc.write('h7ff6b, 0, CR_CSR|A32|D08Byte3);
        acc.write('h7ff6f, 36, CR_CSR|A32|D08Byte3);
        acc.write('h7ff33, 1, CR_CSR|A32|D08Byte3);
        acc.write('h7fffb, 'h10, CR_CSR|A32|D08Byte3); /* enable module (BIT_SET = 0x10) */

        acc.set_default_modifiers(A32 | D32 | SINGLE);
    endtask

    task automatic WbWrite(uint32_t address, uint32_t data, string label = "");
        uint64_t data64 = data;
        
        if (label != "")
            $display(label);
        acc.write(address, data64, A32|SINGLE|D32);
        $display("[0x%x]: 0x%x", address, data);
        #1us; // not to issue commands too fast, to better simulate real behaviour
    endtask
    
    task automatic WbRead(uint32_t address, output uint32_t data, input string label = "");
        uint64_t data64;

        if (label != "")
            $display(label);
        acc.read(address, data64, A32|SINGLE|D32);
        $display("[0x%x]: 0x%x", address, data64[31:0]);
        data = data64[31:0];
        #1us; // not to issue commands too fast, to better simulate real behaviour
    endtask
    
    task automatic TestSdb;
        uint64_t d;
        uint32_t base;
        uint32_t address;
        int i;
        
        base = `WB_SVEC_VME_ADDRESS;
        $display("Carrier SDB:");
        for (i=0; i<64; i = i+4) begin
            address = base + i;
            acc.read(address, d, A32|SINGLE|D32);
            $display("[0x%x]: 0x%x", address, d[31:0]);
        end

        base = `WB_FFPG_BASE + 64*0;
        $display("FFPG SDB:");
        for (i=0; i<64; i = i+4) begin
            address = base + i;
            acc.read(address, d, A32|SINGLE|D32);
            $display("[0x%x]: 0x%x", address, d[31:0]);
        end
    endtask
    
    task automatic TestFfpgCsr;
        uint64_t d;
        uint32_t base;
        uint32_t address;

        address = `WB_FFPG_CSR_REG(`ADDR_FFPG_STATUS);
        $display("FFPG Status reg.:");
        acc.read(address, d, A32|SINGLE|D32);
        $display("[0x%x]: 0x%x", address, d[31:0]);

        address = `WB_FFPG_CSR_REG(`ADDR_FFPG_CONTROL);
        $display("Control reg. write:");
        d = 4;
        acc.write(address, d, A32|SINGLE|D32);
        $display("[0x%x]: 0x%x", address, d[31:0]);

        address = `WB_FFPG_CSR_REG(`ADDR_FFPG_STATUS);
        $display("FFPG Status reg.:");
        acc.read(address, d, A32|SINGLE|D32);
        $display("[0x%x]: 0x%x", address, d[31:0]);
    endtask
    
    task automatic TestReal;
        `include "z:/wb_trace.svh"
        #1ms;
        WbRead(`WB_FFPG_CSR_REG(`ADDR_FFPG_DEBUG), dataOut, "Debug");
    endtask
    
    task automatic TestAd9512Sync;
        uint32_t data;

        WbWrite(`WB_FFPG_CSR_REG(`ADDR_FFPG_CONTROL), 'h200);
        WbRead(`WB_FFPG_CSR_REG(`ADDR_FFPG_CONTROL), data);
        #200us;
        WbRead(`WB_FFPG_CSR_REG(`ADDR_FFPG_CONTROL), data);
    endtask
    
    task automatic WaitUntilRead;
        uint32_t data;
        
        do
            WbRead(`WB_FFPG_CSR_REG(`ADDR_FFPG_STATUS), data);
        while (data & 'b1111);
    endtask
    
    initial begin
        acc = new(VME);

        #10us; // for PLL lock and reset settle
        #20us;
        init_vme64x_core;
        
        WaitUntilRead;
        
        // TestSdb;
        // TestFfpgCsr;
        // TestReal;
        TestAd9512Sync;
        TestAd9512Sync;
        
        // uint64_t blt_addr[];
        // uint64_t blt_data[];
        // $display("Write data to DDR in BLT\n");
        // blt_addr = {'h3000};
        // blt_data = {'h1, 'h2, 'h3, 'h4, 'h5, 'h6, 'h7, 'h8 ,'h9, 'hA};
        // acc.writem(blt_addr, blt_data, A32|BLT|D32, result);

        // $display("Read data from DDR in BLT");
        // blt_data = {};
        // acc.readm(blt_addr, blt_data, A32|BLT|D32, result);

        // for(i=0; i<10; i++) begin
            // $display("Data %d: 0x%x\n", i, blt_data[i]);
        // end

        $stop;
    end

endmodule
